package mvcsSkeleton.models
{
	import flash.events.IEventDispatcher;

	import mvcsSkeleton.controllers.events.StatisticsEvent;
	import mvcsSkeleton.models.api.IStatistics;
	import mvcsSkeleton.models.vos.ShapeVO;

	import mx.collections.ArrayCollection;

	public class StatisticsModel implements IStatistics
	{
		[Inject]
		public var dispatcher:IEventDispatcher;

		private var _shapesCollection:ArrayCollection = new ArrayCollection();

		public function StatisticsModel()
		{
		}

		public function get shapesCollection():ArrayCollection
		{
			return _shapesCollection ||= new ArrayCollection();
		}

		public function set shapesCollection(value:ArrayCollection):void
		{
			_shapesCollection = value;
		}

		public function addElement(shapeVO:ShapeVO):void
		{
			shapesCollection.addItem(shapeVO);
			shapesCollection.refresh();
			dispatcher.dispatchEvent(new StatisticsEvent(StatisticsEvent.STATISTICS_RESULTS, doSomeMath()));
		}

		public function removeElement(shapeVO:ShapeVO):void
		{
			if (shapesCollection.length == 0)
				return;

			if (shapeVO.shapeId == "all")
			{
				shapesCollection.removeAll();
				shapesCollection.refresh();
				dispatcher.dispatchEvent(new StatisticsEvent(StatisticsEvent.STATISTICS_RESULTS, doSomeMath()));
				return;
			}

			var index:int = getIndexById(shapeVO.shapeId)
			if (index == -1)
				return;

			shapesCollection.removeItemAt(index);
			shapesCollection.refresh();

			dispatcher.dispatchEvent(new StatisticsEvent(StatisticsEvent.STATISTICS_RESULTS, doSomeMath()));
		}

		public function doSomeMath():Array
		{
			var payload:Array = [];
			payload["averageSize"] = "";
			payload["medianSize"] = "";
			payload["collection"] = shapesCollection;

			if (shapesCollection.length > 0)
			{
				var sizes:Array = [];
				for each (var shapeVO:ShapeVO in shapesCollection)
				{
					sizes.push(shapeVO.shapeSize);
				}
				sizes.sort(Array.NUMERIC);

				payload["averageSize"] = average(sizes).toFixed(0);
				payload["medianSize"] = median(sizes).toFixed(0);
			}
			return payload;
		}

		private function getIndexById(id:String):int
		{
			for (var i:int = 0; i < shapesCollection.length; i++)
			{
				if (shapesCollection[i].shapeId == id)
					return i;
			}
			return -1;
		}

		private function getMedian(arr:Array):Number
		{
			if (arr.length % 2 == 0)
			{
				var a:Number = arr[int(arr.length / 2) - 1];
				var b:Number = arr[int(arr.length / 2)];

				return (a + b) / 2;
			}

			return arr[int(arr.length / 2)];
		}

		private function average(numbers:Array):Number
		{
			var sum:Number = 0;
			for each (var n:Number in numbers)
			{
				sum += n;
			}
			return sum / numbers.length;
		}

		private function median(numbers:Array):Number
		{
			var middle:Number = numbers.length / 2;
			if (middle is int)
			{
				return numbers[middle];
			}
			else
			{
				return (numbers[int(middle)] + numbers[int(middle) + 1]) / 2;
			}
		}
	}
}
